#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2008 Fabricio Silva

"""
Alternative au fichier systeme.f
(pb de f2py sur le portable dell)
"""

import numpy as np

def version():
    return "Grandeurs dimensionnées - version python."

class param:
    ktable = 0.
    wr, qr, k, w = 0., 0., 0., 0.
    rho =0.
    def __init__(self):
        pass


def systeme(inc,t,sn,Cn,pmpts,ypts):
    """
    evaluate the derivative of vector x at time t
    with complex modes (sn,Cn). Used for the calculation
    of auto-oscillations in resonator-valve coupled system.
    """
    a = param()
    # Reconstitution de la pression à partir des parties réelles.
    p = 2.*inc[:-2:2].sum()
    y = inc[-2]
    dy = inc[-1]
    pm = profilevolution(t, pmpts)
    yr = profilevolution(t, ypts)
    u = couplage(p, y, pm, a.rho, a.w)
    Dinc = np.zeros(len(inc), dtype=float)

    # Partie acoustique
    pn = inc[:-2:2]+1.j*inc[1:-2:2]
    dpn = Cn*u+sn*pn
    Dinc[0:-2:2] = dpn.real
    Dinc[1:-2:2] = dpn.imag

    # Partie anche
    Dinc[-2] = inc[-1]
    Dinc[-1] = -a.qr*a.wr*dy-a.wr**2*(y-yr)+a.wr**2/a.k*(p-pm)
    if y<0: Dinc[-1] -= a.wr**2*a.ktable*y
    return Dinc

def jacobien(inc,t,sn,Cn,pmpts,ypts):
    """
    evaluate the jacobian of vector x at time t
    with complex modes (sn,Cn). Used for the calculation
    of auto-oscillations in resonator-valve coupled system.
    """
    a = param()
    # Reconstitution de la pression à partir des parties réelles.
    p = 2.*inc[:-2:2].sum()
    y = inc[-2]
    dy = inc[-1]
    pm = profilevolution(t, pmpts)
    yr = profilevolution(t, ypts)
    u = couplage(p, y, pm, a.rho, a.w)
    dupn = -u/(pm-p)
    duy = u/y
    Jinc = np.zeros((len(inc), len(inc)), dtype=float)
    Jinc[0:-2:2, 0:-2:2] = Cn.real*dupn
    Jinc[0:-2:2, 1:-2:2] = 0
    Jinc[1:-2:2, 0:-2:2] = Cn.imag*dupn
    Jinc[1:-2:2, 1:-2:2] = 0
    for ind in xrange(len(inc)/2-1):
        Jinc[2*ind, 2*ind] += sn[ind].real
        Jinc[2*ind, 2*ind+1] -= sn[ind].imag
        Jinc[2*ind+1, 2*ind] += sn[ind].imag
        Jinc[2*ind+1, 2*ind+1] += sn[ind].real
        
    Jinc[0:-2:2,-2] = Cn.real*duy
    Jinc[1:-2:2,-2] = Cn.imag*duy
    
    Jinc[0:-2:2,-1] = 0
    Jinc[1:-2:2,-1] = 0
    
    Jinc[-2, 0:-2:2] = 0
    Jinc[-2, 1:-2:2] = 0
    
    Jinc[-1, 0:-2:2] = 2*a.wr**2/a.k
    Jinc[-1, 1:-2:2] = 0
    
    Jinc[-2, -2] = 0
    Jinc[-2, -1] = 1
    Jinc[-1, -2] = -a.wr**2
    if y<0: Jinc[-1, -2] -= a.wr**2*a.ktable
    Jinc[-1, -1] -= -a.wr*a.qr
    
    return Jinc

def couplage(p, y, pm, rho, w):
    """
    evaluates the volume flow entering throught the reed channel
    knowing the pressure difference and the opening of the reed.
    """
    p = np.atleast_1d(p)
    y = np.atleast_1d(y)
    if len(pm)==1 : pm = np.ones_like(p)*pm
    u = np.zeros_like(p)
    
    idx = (pm>p)
    u[idx] = w*y[idx]*np.sqrt(pm[idx]-p[idx])*np.sqrt(2./rho)
    
    idx = (pm<p)
    u[idx] = -w*y[idx]*np.sqrt(-pm[idx]+p[idx])*np.sqrt(2./rho)
    
    u[(y<0)] = 0
    return u

def profilevolution(t, pts):
    """
    evaluates the value of a slowly varying quantity
    according to a curve parametrized with control points.
    The control points are assumed to be inflexion ponts.
    """
    t = np.atleast_1d(t)
    x = np.zeros(len(t), dtype=float)
    x[t<=pts[0,0]] = pts[1,0]
    x[t>pts[0,-1]] = pts[1,-1]
    for ind in xrange(pts.shape[1]-1):
        idx = ((t>pts[0,ind]) & (t<=pts[0,ind+1]))
        tidx = t[idx]
        dt = pts[0,ind+1]-pts[0,ind]
        f0, f1 = pts[1, ind:ind+2]
        fp0, fp1 = pts[2, ind:ind+2]
        w = (tidx-pts[0,ind])/dt
        a5 = -3*dt*fp1-3*dt*fp0+6*f1-6*f0
        a4 = 7*dt*fp1+8*dt*fp0-15*f1+15*f0
        a3 = -4*dt*fp1-6*dt*fp0+10*f1-10*f0
        a2 = 0
        a1 = dt*fp0
        a0 = f0
        x[idx] += a5*w**5+a4*w**4+a3*w**3+a2*w**2+a1*w+a0
    return x

