#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Quickref

Install the package (install command as root):  # python setup.py install
Build for development:                          $ python setup.py build
Generate the documentation:                     $ python setup.py doc
"""

import os, sys
import info
from distutils.core import setup
from distutils.extension import Extension
import numpy as np

commands = {}
name = info.name

if os.path.exists('MANIFEST'):
    os.remove('MANIFEST')

if sys.version_info < (2,4):
    raise SystemExit("Moreesc requires Python 2.4 or later.")

# defining the C extensions (generated by Cython)
kw = dict(
        include_dirs = ['src', info.name, '.', np.get_include()],
        extra_compile_args=["-O3"],
        extra_link_args=["-O3"])
lExtensions = [
    Extension("c_profiles", [os.path.join(info.name, "c_profiles.c"),], **kw),
    Extension("c_acous_sys", [os.path.join(info.name, "c_acous_sys.c"),], **kw),
    Extension("c_mech_sys", [os.path.join(info.name, "c_mech_sys.c"),], **kw),
    Extension("c_mech_sys_controller", [os.path.join(info.name, "c_mech_sys_controller.c"),], **kw),
    Extension("c_coupling", [os.path.join(info.name, "c_coupling.c"),], **kw),
]

setup_kwargs = dict([[k, getattr(info, k)] for k in dir(info)
    if not(k.startswith('__'))])
setup_kwargs.pop('copyright')

if __name__ == "__main__":
    if 'doc' in sys.argv[1]:
        try:
            from sphinx.setup_command import BuildDoc
            commands['doc']=BuildDoc
        except ImportError:
            raise ImportError, "Can't 'doc' without Sphinx."

    if 'cython' in sys.argv[1]:
        # Default Cython compilation is not recommended
        # http://docs.cython.org/src/userguide/source_files_and_compilation.html#distributing-cython-modules
        try:
            from Cython.Distutils import build_ext
            commands['build_ext'] = build_ext
        except:
            print "You don't seem to have Cython installed. Please get a"
            print "copy from www.cython.org and install it"
            sys.exit(1)
        
        def cythonizer(fname):
            if fname.endswith('.c') and os.path.basename(fname).startswith('c_'):
                return fname[:-1] + 'pyx'
            return fname
        for ext in lExtensions:
            ext.sources = [cythonizer(tmp) for tmp in ext.sources]
            ext.pyrex_directives = {
                "warn.unused": False,
                "warn.unused_arg": False,
                "warn.unused_result": False}
        sys.argv[1] = 'build'
    # Cython files to distribute
    cython_files = [os.path.join(info.name, tmp) for tmp in os.listdir(info.name)
        if ((os.path.splitext(tmp)[-1] in ('.pyx', '.pxd'))
         or (tmp.endswith('.c') and os.path.exists(tmp.replace('.c', '.pyx'))))]

    if len(sys.argv)>1:
        setup(
            provides=[info.name],
            packages=[info.name, info.name + '.tests'],
#            package_dir={'': 'moreesc'},
#            package_data={'moreesc': ['moreesc/tests/test_*.py']},
            data_files=[
                ('', ['info.py',]),
                ('data', ['data/Logo_serpent.png',]),
                (info.name, cython_files)],
            ext_package=info.name,
            ext_modules=lExtensions,
            cmdclass = commands,
            **setup_kwargs)
